//
// This file was modified from the original implementation of altera_avalon_remote_update_cycloneiii.v
//

/*****************************************************************************/
// altera_avalon_remote_update_cycloneiii.v
//
// This verilog file is a lightweight wrapper for the Cyclone III remote_update
// block.  Using this wrapper, the remote update block can be imported into 
// SOPC Builder using Component Editor.
//  
// Modifications made with this wrapper:
//
// 1.) The data_in and data_out ports of the remote update block have been 
//     expanded to 32-bits to comply with SOPC Builder 
// 2.) A register has been added so that the reconfig and reset_timer control 
//     bits of the remote update block can be controlled from an Avalon master
//     (such as NiosII)
// 3.) The param and read_source lines have been combined into a single address
//     bus so each register from each configuration state has a unique address
//
/*****************************************************************************/


module rsu_cyclone4 (
  clk,
  reset,
  writedata,
  readdata,
  address,
  chipselect,
  write,
  read,
  waitrequest );
  
  
  input clk;
  input reset;
  input [31:0] writedata;
  output [31:0] readdata;
  input [5:0] address;
  input chipselect;
  input write;
  input read;
  output waitrequest;
  

  /************************************************************************
  **  Address[2:0] description
  *************************************************************************

    000 - Master State Machine Current State Mode - width of 2
    001 - Force early CONF_DONE (Cd_early) check - width of 1
    010 - Watchdog Timeout Value - width of 12 when reading and
          width of 29 when writing
    011 - Watchdog Enable - width of 1
    100 - Boot Address- width of 24 when reading and width of 22
          when writing
    101 - (illegal value)
    110 - Force the internal oscillator as startup state machine clock
          (Osc_int) option bit - width of 1
    111 - Reconfiguration trigger conditions - width of 5
    
  *************************************************************************
  **  Address[4:3] description
  *************************************************************************
    
    00 Current State Contents in Status Register
    01 Previous State Register 1 Contents in Status Register
    10 Previous State Register 2 Contents in Status Register
    11 Illegal
    
  
  *************************************************************************
  **  Address[5] description
  *************************************************************************

		Enables access to control_status_reg
		  -Used to control 'reconfig' and 'reset_timer' signals from Avalon
		  
  ************************************************************************/
`ifdef QUARTUS__SIMGEN
  assign waitrequest = 1'b0;
  assign readdata = 32'b0;
`else

  wire [2:0] param_wire = address[2:0];
  wire [1:0] read_source = address[4:3];
  wire read_param;
  wire write_param;
  wire [21:0] data_in = writedata[21:0];
  wire [28:0] data_out;
  wire reconfig;
  wire reset_timer;
  wire reset_n = ~reset;
  wire waitrequest;
  wire busy;

  reg readorwritelastclock;
  reg busylastclock;
  reg [31:0] readdata;
  reg [31:0] control_status_reg;

  assign read_param = chipselect & read & ~address[5] & ~readorwritelastclock;
  assign write_param = chipselect & write & ~address[5] & ~readorwritelastclock;
  assign waitrequest = (( busy ^ (( write | read ) & ~readorwritelastclock )) | busylastclock );

  /*****************************************************************************/
  // General Purpose Register - Read/Writeable by Avalon master
  // Currently used to provide bus access to Reconfig / Watchdog Reset bits
  //  Offset = 0x10000
  /*****************************************************************************/
  always @(posedge clk or negedge reset_n)
  if (~reset_n)
    control_status_reg <= 0;
  else if ( chipselect & write & ( address == 6'b100000 ))
    control_status_reg <= writedata;
  else
    control_status_reg <= control_status_reg;


  // Register read logic
  always @(posedge clk or negedge reset_n)
    if( ~reset_n )
      readdata <= 0;
    else if( chipselect & read & ~address[5] )
      readdata <= {3'b000, data_out};
    else if( chipselect & read & ( address == 6'b100000 ))
      readdata <= control_status_reg;
    else
      readdata <= readdata;
      
//  assign readdata = address[5] ? control_status_reg: {4'b0000, data_out} ;

  // Delayed 'OR' of the read and write signals
  // Used to pulse the read_param/write_param signals for just one clock cycle
  always @(posedge clk or negedge reset_n)
  if (~reset_n)
    readorwritelastclock <= 0;
  else 
    readorwritelastclock <= ( read | write );

  // Delayed busy signal
  // Used to hold waitrequest for one more cycle
  always @(posedge clk or negedge reset_n)
  if (~reset_n)
    busylastclock <= 0;
  else 
    busylastclock <= busy;

  

  assign reconfig = control_status_reg[0];
  assign reset_timer = control_status_reg[1];

  /*****************************************************************************/
  // Instantiation of remote update funciton.
  /*****************************************************************************/        
altremote_update_cyclone4	altremote_update_cyclone4_inst (
	.clock ( clk ),
	.data_in ( writedata[21:0] ),
	.param ( param_wire ),
	.read_param ( read_param ),
	.read_source ( read_source ),
	.reconfig ( reconfig ),
	.reset ( reset ),
	.reset_timer ( reset_timer ),
	.write_param ( write_param ),
	.busy ( busy ),
	.data_out ( data_out )
	);

`endif

endmodule
