# This is the boot address for the Application Image
# 0x40000 correlates to byte address of 0x100000 for the start location within the FLASH device.
# From the documentation: RSU controller which provides an address register for us that contains 
# the upper 22 bits of a 24 bit address to fetch the next configuration from. 
# So application images must be 4 byte aligned in the flash
set BootAddrVal 0x40000

# Base addresses from Qsys system
set RSU_Base 0x00000000
set HW_VerAddr 0x00000100

# Addresses within RSU component for accessing Reconfig Source, AI Boot Addr and reconfig Bit.
# Shift left by 2 in order to accomodate byte vs word addressing through System Console.

# **** All addresses below are used for MSM Mode FACTORY ****

# Read Master State Machine Current State read_source = 00b; param = 000b;
set RdMstrStMachCurrStMdAddr  [expr $RSU_Base + [expr 0x0  << 2]]

# Read Past Status 1 Reconfiguration Trigger Condition Source contents in status register read_source = 01b; param = 111b;
set RdPastSt1_ReconfigSrcAddr  [expr $RSU_Base + [expr 0xF  << 2]]

# Read Past Status 1 Reconfiguration Trigger Condition Source contents in status register read_source = 10b; param = 111b;
set RdPastSt2_ReconfigSrcAddr  [expr $RSU_Base + [expr 0x17  << 2]]

# Reconfig control address is address[5] and goes directly to the reconfig signal of 
# the alremote_update_cyclone4 function in rsu_cyclone4.v.  read_source and param are not used when
# writing the reconfig port of the altremote_update_cyclone4 function.
set ReconfigCtrlAddr [expr $RSU_Base + [expr 0x20 << 2]]

# Read Factory Boot address read_source = 00b; param = 100b;
set RdFactBootAddrAddr     [expr $RSU_Base + [expr 0x4  << 2]]

# Read Past Status 1 Boot Address read_source = 01b; param = 100b;
set RdPastSt1BootAddrAddr     [expr $RSU_Base + [expr 0xC  << 2]]

# Read Past Status 2 Boot Address read_source = 10b; param = 100b;
set RdPastSt2BootAddrAddr     [expr $RSU_Base + [expr 0x14  << 2]]

# Write Application Boot address read_source = 00b; param = 100b;
set WrAppBootAddrAddr     [expr $RSU_Base + [expr 0x4  << 2]]

# Read Boot Address read_source = 11b; param = 100b;
set RdBootAddrAddr     [expr $RSU_Base + [expr 0x1C  << 2]]

# Read Watchdog Enable bit Address read_source = 11b; param = 011b;
set RdWdogEnAddr [expr $RSU_Base + [expr 0x1B << 2]]

# Write Watchdog Enable bit Address read_source = 00b; param = 011b;
set WrWdogEnAddr [expr $RSU_Base + [expr 0x3 << 2]]


puts [concat "RSU Base: " [format {%08x} $RSU_Base]]
puts [concat "Read Master State Machine Current State Mode Addr: " [format {%08x} $RdMstrStMachCurrStMdAddr]]
puts [concat "Read Past Status 1 Reconfiguration Trigger Condition Source Addr: " [format {%08x} $RdPastSt1_ReconfigSrcAddr]]
puts [concat "Read Past Status 2 Reconfiguration Trigger Condition Source Addr : " [format {%08x} $RdPastSt2_ReconfigSrcAddr]]
puts [concat "Reconfig address used as control switch to application image: " [format {%08x} $ReconfigCtrlAddr]]
puts [concat "Read Factory Boot Address Addr: " [format {%08x} $RdFactBootAddrAddr]]
puts [concat "Read Past Status 1 Boot Address Addr: " [format {%08x} $RdPastSt1BootAddrAddr]]
puts [concat "Read Past Status 2 Boot Address Addr: " [format {%08x} $RdPastSt2BootAddrAddr]]
puts [concat "Write Application Boot Address Addr: " [format {%08x} $WrAppBootAddrAddr]]
puts [concat "Read Boot Address Addr: " [format {%08x} $RdBootAddrAddr]]
puts [concat "Read Watchdog Enable bit Addr: " [format {%08x} $RdWdogEnAddr]]
puts [concat "Write Watchdog Enable bit Addr: " [format {%08x} $WrWdogEnAddr]]

puts "\n"

# Open the 1st master in the system - Should be the JTAG-to-Avalon Master
set m [ lindex [ get_service_paths master ] 0 ]
open_service master $m

# Read HWVERSION
set HW_ver [master_read_32 $m $HW_VerAddr 1]
puts [concat "HW Version:" $HW_ver]

# Read Master State Machine Current State Mode
set RdMstrStMachCurrStMd [master_read_32 $m $RdMstrStMachCurrStMdAddr 1]
puts [concat "Master State Machine (MSM) Current State Mode:" $RdMstrStMachCurrStMd]

# Read Past Status 1 Reconfiguration Trigger Condition Source
set ReconfigSourceRead1 [master_read_32 $m $RdPastSt1_ReconfigSrcAddr 1]
puts [concat "Past Status 1 Reconfig Trigger Condition Source:" $ReconfigSourceRead1]

# Read Past Status 2 Reconfiguration Trigger Condition Source
set ReconfigSourceRead2 [master_read_32 $m $RdPastSt2_ReconfigSrcAddr 1]
puts [concat "Past Status 2 Reconfig Trigger Condition Source:" $ReconfigSourceRead2]

# Read Factory Boot Address
set FactoryBootAddr [master_read_32 $m $RdFactBootAddrAddr 1]
puts [concat "Factory Boot Address:" $FactoryBootAddr]

# Read Past Status 1 Boot Address
set PastSt1BootAddr [master_read_32 $m $RdPastSt1BootAddrAddr 1]
puts [concat "Past Status 1 Boot Address:" $PastSt1BootAddr]

# Read Past Status 2 Boot Address
set PastSt2BootAddr [master_read_32 $m $RdPastSt2BootAddrAddr 1]
puts [concat "Past Status 2 Boot Address:" $PastSt2BootAddr]

# Write Application Boot Address
puts [concat "Setting Application Boot Address: " [format {%08x} $BootAddrVal]]
master_write_32 $m $WrAppBootAddrAddr $BootAddrVal

# Read back boot address
set BootAddrReadBack [master_read_32 $m $RdBootAddrAddr 1]
puts [concat "Read Back Boot Address:" $BootAddrReadBack]

puts "\n"

# Read Watchdog Enable bit
set RdWdogEn [master_read_32 $m $RdWdogEnAddr 1]
puts [concat "Watchdog Enable bit:" $RdWdogEn]

# Write Watchdog Enable bit to 0 and turn off
puts [concat "Write Watchdog Enable bit to 0"]
master_write_32 $m $WrWdogEnAddr 0x0

# Read Watchdog Enable bit
set RdWdogEn [master_read_32 $m $RdWdogEnAddr 1]
puts [concat "Watchdog Enable bit:" $RdWdogEn]

puts "\n"


# Write RECONFIG
puts [concat "Writing Reconfig Bit..."]
master_write_32 $m $ReconfigCtrlAddr 0x1
